==> eslint-plugin-react-props-in-state-1.0.1.tgz: package/index.js <==
const FE_TYPES = [
  'FunctionExpression',
  'ArrowFunctionExpression'
]

const isObject = (any) => {
  return any && typeof any === 'object'
}

const state = {
  imports: [],
  reports: [],
  reactComponents: []
}

const registerImport = (node) => {
  state.imports = [
    ...state.imports,
    ...node.specifiers.map((specifier) => {
      return {
        name: specifier.local.name,
        type: specifier.type,
        source: node.source.value
      }
    })
  ]
}

const isReactComponentDestructed = (node) => {
  // Case of extending with destructed property like:
  // class Button extends Component
  const sup = node.superClass
  const componentClass = state.imports.find((current) => {
    const nameMatch = sup && current.name === sup.name && sup.name === 'Component'
    const isSpecifier = current.type === 'ImportSpecifier'
    const isFromReact = current.source === 'react'
    return nameMatch && isSpecifier && isFromReact
  })
  return componentClass !== undefined
}

const isReactComponentDefault = (node) => {
  // Case of extending with default react export like:
  // class Button extends React.Component
  const sup = node.superClass
  const fromReact = sup && sup.type === 'MemberExpression'
  const defaultReact = state.imports.find((current) => {
    return current.type === 'ImportDefaultSpecifier'
  })
  if (fromReact && defaultReact) {
    const supObj = sup.object
    const supProp = sup.property
    if (supObj && supProp) {
      const isReact = supObj.type === 'Identifier' && supObj.name === defaultReact.name
      const isComponent = supProp.type === 'Identifier' && supProp.name === 'Component'
      return isReact && isComponent
    }
  }
  return false
}

const isReactComponent = (node) => {
  return isReactComponentDestructed(node) || isReactComponentDefault(node)
}

const registerReactComponent = (node) => {
  if (isReactComponent(node)) {
    state.reactComponents = [
      ...state.reactComponents,
      node
    ]
  }
}

const getPropName = (node) => {
  const tokenTypes = {
    Literal: (node) => {
      return node.key.value
    },
    Identifier: (node) => {
      return node.key.name
    }
  }

  return tokenTypes.hasOwnProperty(node.key.type)
    ? tokenTypes[node.key.type](node)
    : undefined
}

const getClassMethodNames = (node) => {
  return getClassMethods(node).map((method) => {
    return getPropName(method)
  })
}

const getClassProps = (node) => {
  return node.body.body.reduce((props, prop) => {
    return ~FE_TYPES.indexOf(prop.value.type) ? props : [...props, prop]
  }, [])
}

const getClassPropNames = (node) => {
  return getClassProps(node).map((prop) => {
    return getPropName(prop)
  })
}

const getClassMethods = (node) => {
  return node.body.body.reduce((methods, prop) => {
    switch (prop.type) {
      case 'ClassProperty': {
        if (~FE_TYPES.indexOf(prop.value.type)) {
          return [...methods, prop]
        }
        break
      }
      case 'MethodDefinition': {
        return [...methods, prop]
      }
    }
    return methods
  }, [])
}

const getFunctionParams = (node) => {
  return node.params ? node.params.map((param) => (param.name)) : []
}

const tokenTypedParent = (node) => {
  if (!isObject(node)) {
    return null
  }

  const isTyped = typeof node.type === 'string'
  return isTyped
    ? node
    : tokenTypedParent(node.parent)
}

const flattenTokens = (node) => {
  return flattenTree(node).filter((value) => {
    return typeof value.type === 'string'
  })
}

const hasUsageOfNextProps = (node) => {
  return flattenTokens(node).filter((token) => {
    return token.name === 'nextProps' && token.type === 'Identifier'
  }).some((token) => {
    return token.closestParentToken.type !== 'MemberExpression'
  })
}

const hasDirectUsageOfThisPropsIn = (node) => {
  return flattenTokens(node).filter((token) => {
    return token.name === 'props' && token.type === 'Identifier'
  }).some((token) => {
    const parentObject = token.closestParentToken.object
    return isObject(parentObject) && parentObject.type === 'ThisExpression'
  })
}

const indirectUsagesOfThisPropsIn = (node, component, depth = 5) => {
  const componentMethods = getClassMethods(component)
  const callExpressions = instanceMethodsCallExpressionsIn(node)
  return callExpressions.map((method) => {
    return {
      methodCallNode: method,
      methodInitNode: componentMethods.find((componentMethod) => {
        return getPropName(componentMethod) === callExpressionName(method)
      })
    }
  }).filter((method) => {
    return method.methodInitNode !== undefined
  }).filter((method) => {
    return hasUsageOfThisPropsIn(method.methodInitNode, component, depth)
  }).map((method) => {
    return method.methodCallNode
  })
}

const indirectUsagesOfThisPropsInConstructor = (constructorMethod, component) => {
  return flattenTokens(constructorMethod.value.body).filter((token) => {
    return token.type === 'Identifier' && token.name === 'state'
  }).filter(stateAssignment).reduce((usages, token) => {
    const stateAssignment = token.closestParentToken.closestParentToken.closestParentToken.right
    return [...usages, ...indirectUsagesOfThisPropsIn(stateAssignment, component)]
  }, [])
}

const hasIndirectUsageOfThisPropsIn = (node, component, depth) => {
  const componentMethods = getClassMethods(component)
  return instanceMethodsCallExpressionsNamesIn(node).map((method) => {
    return componentMethods.find((componentMethod) => {
      return getPropName(componentMethod) === method
    })
  }).filter((method) => {
    return method !== undefined
  }).some((method) => {
    return hasUsageOfThisPropsIn(method, component, depth)
  })
}

const hasUsageOfThisPropsIn = (node, component, depth = 5) => {
  if (hasDirectUsageOfThisPropsIn(node)) {
    return true
  }
  return (depth || depth === null)
    ? hasIndirectUsageOfThisPropsIn(node, component, --depth)
    : false
}

const instanceMethodsCallExpressionsIn = (node) => {
  return flattenTokens(node).filter((token) => {
    return token.type === 'CallExpression'
      && isObject(token.callee)
      && isObject(token.callee.object)
      && token.callee.object.type === 'ThisExpression'
  }).map((token) => {
    return token.callee.property
  })
}

const callExpressionName = (node) => {
  const nameProps = {
    Literal: 'value',
    Identifier: 'name'
  }
  return nameProps.hasOwnProperty(node.type) ? node[nameProps[node.type]] : null
}

const instanceMethodsCallExpressionsNamesIn = (node) => {
  const nameProps = {
    Literal: 'value',
    Identifier: 'name'
  }
  return instanceMethodsCallExpressionsIn(node).reduce((names, callee) => {
    const name = callExpressionName(callee)
    return name
      ? [...names, name]
      : names
  }, [])
}

const flattenTree = (node, parent = null) => {
  const treeElement = Object.assign({}, node, {
    parent,
    closestParentToken: tokenTypedParent(parent)
  })

  return Object.keys(node).reduce((flatten, prop) => {
    const isAllowed = !~['parent'].indexOf(prop)
    const isPublic = prop.charAt(0) !== '_'
    const shouldTraverse = isObject(node[prop]) && isAllowed && isPublic
    return shouldTraverse
      ? [...flatten, ...flattenTree(node[prop], treeElement)]
      : flatten
  }, [treeElement] : [])
}

const stateAssignment = (node) => {
  const thisProp = isObject(node.closestParentToken)
    && isObject(node.closestParentToken.object)
    && node.closestParentToken.type === 'MemberExpression'
    && node.closestParentToken.object.type === 'ThisExpression'
  if (thisProp) {
    const memberExp = node
      .closestParentToken
      .closestParentToken
    const thisPropMemberExp = memberExp.type === 'MemberExpression'
    if (thisPropMemberExp) {
      const assignment = node
        .closestParentToken
        .closestParentToken
        .closestParentToken
      const isAssigned = assignment.type === 'AssignmentExpression'
        && assignment.left.range === memberExp.range
      return isAssigned
    }
  }
  return false
}

const initialStateNodes = (node) => {
  return ['state', 'constructor'].map((name) => {
    return node.body.body.find((prop) => {
      return getPropName(prop) === name
    })
  })
}

const usePropsInState = (state, component) => {
  return state ? hasUsageOfThisPropsIn(state, component) : false
}

const usePropsInConstructor = (constructorMethod, component) => {
  return flattenTokens(constructorMethod.value.body).filter((token) => {
    return token.type === 'Identifier' && token.name === 'state'
  }).filter(stateAssignment).some((token) => {
    const stateAssignment = token.closestParentToken.closestParentToken.closestParentToken.right
    return hasUsageOfThisPropsIn(stateAssignment, component)
  })
}

const useProps = (node) => {
  const [stateProp, constructorMethod] = initialStateNodes(node)
  return usePropsInState(stateProp, node) || usePropsInConstructor(constructorMethod, node) || false
}

const implicitPropsUses = (node) => {
  const [stateProp, constructorMethod] = initialStateNodes(node)
  const statePropImpicit = stateProp
    ? indirectUsagesOfThisPropsIn(stateProp, node)
    : []
  const constructorMethodImplicit = constructorMethod
    ? indirectUsagesOfThisPropsInConstructor(constructorMethod, node)
    : []
  const nodes = [...statePropImpicit, ...constructorMethodImplicit]
  return nodes.length ? nodes : null
}

const checks = [{
  check: (component) => {
    return ~getClassMethodNames(component).indexOf('componentWillReceiveProps') ? null : component
  },
  error: 'missing componentWillReceiveProps() method'
}, {
  check: (component) => {
    // Check if componentWillReceiveProps() exists...
    const componentWillReceiveProps = getClassMethods(component).find((method) => {
      return getPropName(method) === 'componentWillReceiveProps'
    })
    if (!componentWillReceiveProps) {
      return component
    }
    // ...then check if there is an argument `nextProps`...
    const hasNextProps = getFunctionParams(componentWillReceiveProps.value)[0] === 'nextProps'
    if (!hasNextProps) {
      return componentWillReceiveProps
    }
    // ...and finally check if `nextProps` is used inside of componentWillReceiveProps.
    if (!hasUsageOfNextProps(componentWillReceiveProps.value.body)) {
      return componentWillReceiveProps
    }
    return null
  },
  error: 'missing or unsued nextProps argument of componentWillReceiveProps() method'
}, {
  check: (component) => {
    return implicitPropsUses(component)
  },
  error: 'implicit use of this.props while mutating this.state'
}]

const rules = function(context) {
	return {
    ImportDeclaration(node) {
      registerImport(node)
    },
    ClassDeclaration(node) {
      if (useProps(node)) {
        registerReactComponent(node)
      }
    },
    'Program:exit'() {
      const reports = state.reactComponents.reduce((reports, component) => {
        return [...reports, ...checks.reduce((localReports, check) => {
          const errNode = check.check(component)
          return !errNode ? localReports : [...localReports, {
            node: errNode,
            error: check.error
          }]
        }, [])]
      }, [])
      reports.forEach((report) => {
        const repNodes = Array.isArray(report.node) ? report.node : [report.node]
        repNodes.forEach((node) => {
          context.report(node, report.error)
        })
      })
    }
	}
}

module.exports = {
  rules: { rules }
}

==> eslint-plugin-react-props-in-state-1.0.3.tgz: package/index.js <==
const FE_TYPES = [
  'FunctionExpression',
  'ArrowFunctionExpression'
]

const COMPONENT_CLASSES = [
  'PureComponent',
  'Component'
]

const isObject = (any) => {
  return any && typeof any === 'object'
}

const state = {
  imports: [],
  reports: [],
  reactComponents: []
}

const registerImport = (node) => {
  state.imports = [
    ...state.imports,
    ...node.specifiers.map((specifier) => {
      return {
        name: specifier.local.name,
        type: specifier.type,
        source: node.source.value
      }
    })
  ]
}

const isReactComponentDestructed = (node) => {
  // Case of extending with destructed property like:
  // class Button extends Component
  const sup = node.superClass
  const componentClass = state.imports.find((current) => {
    const nameMatch = sup && current.name === sup.name && ~COMPONENT_CLASSES.indexOf(sup.name)
    const isSpecifier = current.type === 'ImportSpecifier'
    const isFromReact = current.source === 'react'
    return nameMatch && isSpecifier && isFromReact
  })
  return componentClass !== undefined
}

const isReactComponentDefault = (node) => {
  // Case of extending with default react export like:
  // class Button extends React.Component
  const sup = node.superClass
  const fromReact = sup && sup.type === 'MemberExpression'
  const defaultReact = state.imports.find((current) => {
    return current.type === 'ImportDefaultSpecifier'
  })
  if (fromReact && defaultReact) {
    const supObj = sup.object
    const supProp = sup.property
    if (supObj && supProp) {
      const isReact = supObj.type === 'Identifier' && supObj.name === defaultReact.name
      const isComponent = supProp.type === 'Identifier' && ~COMPONENT_CLASSES.indexOf(supProp.name)
      return isReact && isComponent
    }
  }
  return false
}

const isReactComponent = (node) => {
  return isReactComponentDestructed(node) || isReactComponentDefault(node)
}

const registerReactComponent = (node) => {
  if (isReactComponent(node)) {
    state.reactComponents = [
      ...state.reactComponents,
      node
    ]
  }
}

const getPropName = (node) => {
  const tokenTypes = {
    Literal: (node) => {
      return node.key.value
    },
    Identifier: (node) => {
      return node.key.name
    }
  }

  return tokenTypes.hasOwnProperty(node.key.type)
    ? tokenTypes[node.key.type](node)
    : undefined
}

const getClassMethodNames = (node) => {
  return getClassMethods(node).map((method) => {
    return getPropName(method)
  })
}

const getClassProps = (node) => {
  return node.body.body.reduce((props, prop) => {
    return ~FE_TYPES.indexOf(prop.value.type) ? props : [...props, prop]
  }, [])
}

const getClassPropNames = (node) => {
  return getClassProps(node).map((prop) => {
    return getPropName(prop)
  })
}

const getClassMethods = (node) => {
  return node.body.body.reduce((methods, prop) => {
    if (!prop.value) {
    	return methods
    }
    switch (prop.type) {
      case 'ClassProperty': {
        if (~FE_TYPES.indexOf(prop.value.type)) {
          return [...methods, prop]
        }
        break
      }
      case 'MethodDefinition': {
        return [...methods, prop]
      }
    }
    return methods
  }, [])
}

const getFunctionParams = (node) => {
  return node.params ? node.params.map((param) => (param.name)) : []
}

const tokenTypedParent = (node) => {
  if (!isObject(node)) {
    return null
  }

  const isTyped = typeof node.type === 'string'
  return isTyped
    ? node
    : tokenTypedParent(node.parent)
}

const flattenTokens = (node) => {
  return flattenTree(node).filter((value) => {
    return typeof value.type === 'string'
  })
}

const hasUsageOfNextProps = (node) => {
  return flattenTokens(node).filter((token) => {
    return token.name === 'nextProps' && token.type === 'Identifier'
  }).some((token) => {
    return token.closestParentToken.type !== 'MemberExpression' || (
    	token.closestParentToken.type === 'MemberExpression'
      	&& token.closestParentToken.object.range === token.range
    )
  })
}

const hasDirectUsageOfThisPropsIn = (node) => {
  return flattenTokens(node).filter((token) => {
    return token.name === 'props' && token.type === 'Identifier'
  }).some((token) => {
    const parentObject = token.closestParentToken.object
    return isObject(parentObject) && parentObject.type === 'ThisExpression'
  })
}

const indirectUsagesOfThisPropsIn = (node, component, depth = 5) => {
  const componentMethods = getClassMethods(component)
  const callExpressions = instanceMethodsCallExpressionsIn(node)
  return callExpressions.map((method) => {
    return {
      methodCallNode: method,
      methodInitNode: componentMethods.find((componentMethod) => {
        return getPropName(componentMethod) === callExpressionName(method)
      })
    }
  }).filter((method) => {
    return method.methodInitNode !== undefined
  }).filter((method) => {
    return hasUsageOfThisPropsIn(method.methodInitNode, component, depth)
  }).map((method) => {
    return method.methodCallNode
  })
}

const indirectUsagesOfThisPropsInConstructor = (constructorMethod, component) => {
  return flattenTokens(constructorMethod.value.body).filter((token) => {
    return token.type === 'Identifier' && token.name === 'state'
  }).filter(stateAssignment).reduce((usages, token) => {
    const stateAssignment = token.closestParentToken.closestParentToken.closestParentToken.right
    return [...usages, ...indirectUsagesOfThisPropsIn(stateAssignment, component)]
  }, [])
}

const hasIndirectUsageOfThisPropsIn = (node, component, depth) => {
  const componentMethods = getClassMethods(component)
  return instanceMethodsCallExpressionsNamesIn(node).map((method) => {
    return componentMethods.find((componentMethod) => {
      return getPropName(componentMethod) === method
    })
  }).filter((method) => {
    return method !== undefined
  }).some((method) => {
    return hasUsageOfThisPropsIn(method, component, depth)
  })
}

const hasUsageOfThisPropsIn = (node, component, depth = 5) => {
  if (hasDirectUsageOfThisPropsIn(node)) {
    return true
  }
  return (depth || depth === null)
    ? hasIndirectUsageOfThisPropsIn(node, component, --depth)
    : false
}

const instanceMethodsCallExpressionsIn = (node) => {
  return flattenTokens(node).filter((token) => {
    return token.type === 'CallExpression'
      && isObject(token.callee)
      && isObject(token.callee.object)
      && token.callee.object.type === 'ThisExpression'
  }).map((token) => {
    return token.callee.property
  })
}

const callExpressionName = (node) => {
  const nameProps = {
    Literal: 'value',
    Identifier: 'name'
  }
  return nameProps.hasOwnProperty(node.type) ? node[nameProps[node.type]] : null
}

const instanceMethodsCallExpressionsNamesIn = (node) => {
  const nameProps = {
    Literal: 'value',
    Identifier: 'name'
  }
  return instanceMethodsCallExpressionsIn(node).reduce((names, callee) => {
    const name = callExpressionName(callee)
    return name
      ? [...names, name]
      : names
  }, [])
}

const flattenTree = (node, parent = null) => {
  const treeElement = Object.assign({}, node, {
    parent,
    closestParentToken: tokenTypedParent(parent)
  })

  return Object.keys(node).reduce((flatten, prop) => {
    const isAllowed = !~['parent'].indexOf(prop)
    const isPublic = prop.charAt(0) !== '_'
    const shouldTraverse = isObject(node[prop]) && isAllowed && isPublic
    return shouldTraverse
      ? [...flatten, ...flattenTree(node[prop], treeElement)]
      : flatten
  }, [treeElement])
}

const stateAssignment = (node) => {
  const thisProp = isObject(node.closestParentToken)
    && isObject(node.closestParentToken.object)
    && node.closestParentToken.type === 'MemberExpression'
    && node.closestParentToken.object.type === 'ThisExpression'
  if (thisProp) {
    const memberExp = node
      .closestParentToken
      .closestParentToken
    const thisPropMemberExp = memberExp.type === 'MemberExpression'
    if (thisPropMemberExp) {
      const assignment = node
        .closestParentToken
        .closestParentToken
        .closestParentToken
      const isAssigned = assignment.type === 'AssignmentExpression'
        && assignment.left.range === memberExp.range
      return isAssigned
    }
  }
  return false
}

const initialStateNodes = (node) => {
  return ['state', 'constructor'].map((name) => {
    return node.body.body.find((prop) => {
      return getPropName(prop) === name
    })
  })
}

const usePropsInState = (state, component) => {
  return state ? hasUsageOfThisPropsIn(state, component) : false
}

const usePropsInConstructor = (constructorMethod, component) => {
  return flattenTokens(constructorMethod.value.body).filter((token) => {
    return token.type === 'Identifier' && token.name === 'state'
  }).filter(stateAssignment).some((token) => {
    const stateAssignment = token.closestParentToken.closestParentToken.closestParentToken.right
    return hasUsageOfThisPropsIn(stateAssignment, component)
  })
}

const useProps = (node) => {
  const [stateProp, constructorMethod] = initialStateNodes(node)
  return (stateProp ? usePropsInState(stateProp, node) : false)
  	|| (constructorMethod ? usePropsInConstructor(constructorMethod, node) : false)
  	|| false
}

const implicitPropsUses = (node) => {
  const [stateProp, constructorMethod] = initialStateNodes(node)
  const statePropImpicit = stateProp
    ? indirectUsagesOfThisPropsIn(stateProp, node)
    : []
  const constructorMethodImplicit = constructorMethod
    ? indirectUsagesOfThisPropsInConstructor(constructorMethod, node)
    : []
  const nodes = [...statePropImpicit, ...constructorMethodImplicit]
  return nodes.length ? nodes : null
}

const checks = [{
  check: (component) => {
    return ~getClassMethodNames(component).indexOf('componentWillReceiveProps') ? null : component
  },
  error: 'missing componentWillReceiveProps() method'
}, {
  check: (component) => {
    // Check if componentWillReceiveProps() exists...
    const componentWillReceiveProps = getClassMethods(component).find((method) => {
      return getPropName(method) === 'componentWillReceiveProps'
    })
    if (!componentWillReceiveProps) {
      return component
    }
    // ...then check if there is an argument `nextProps`...
    const hasNextProps = getFunctionParams(componentWillReceiveProps.value)[0] === 'nextProps'
    if (!hasNextProps) {
      return componentWillReceiveProps
    }
    // ...and finally check if `nextProps` is used inside of componentWillReceiveProps.
    if (!hasUsageOfNextProps(componentWillReceiveProps.value.body)) {
      return componentWillReceiveProps
    }
    return null
  },
  error: 'missing or unsued nextProps argument of componentWillReceiveProps() method'
}, {
  check: (component) => {
    return implicitPropsUses(component)
  },
  error: 'implicit use of this.props while mutating this.state'
}]

const rules = function(context) {
	return {
    ImportDeclaration(node) {
      registerImport(node)
    },
    ClassDeclaration(node) {
      if (useProps(node)) {
        registerReactComponent(node)
      }
    },
    'Program:exit'() {
      const reports = state.reactComponents.reduce((reports, component) => {
        return [...reports, ...checks.reduce((localReports, check) => {
          const errNode = check.check(component)
          return !errNode ? localReports : [...localReports, {
            node: errNode,
            error: check.error
          }]
        }, [])]
      }, [])
      reports.forEach((report) => {
        const repNodes = Array.isArray(report.node) ? report.node : [report.node]
        repNodes.forEach((node) => {
          context.report(node, report.error)
        })
      })
    }
	}
}

module.exports = {
  rules: { rules }
}
